@@ -13,10 +13,10 @@
 //! use tokio::fs::{self, File};
 //! use tokio_util::io::ReaderStream;
 //! use tower::{Service, ServiceExt, ServiceBuilder, service_fn};
-//! use tower_http::compression::CompressionLayer;
+//! use tower_http::{compression::CompressionLayer, BoxError};
 //!
 //! # #[tokio::main]
-//! # async fn main() -> Result<(), Box<dyn std::error::Error>> {
+//! # async fn main() -> Result<(), BoxError> {
 //! async fn handle(req: Request<Body>) -> Result<Response<Body>, Infallible> {
 //!     // Open the file.
 //!     let file = File::open("Cargo.toml").await.expect("file missing");
@@ -62,113 +62,24 @@
 //! # Ok(())
 //! # }
 //! ```
+//!
 
-use crate::compression_utils::AcceptEncoding;
-use http::{header, HeaderMap, HeaderValue};
+pub mod predicate;
 
 mod body;
 mod future;
 mod layer;
 mod service;
 
+#[doc(inline)]
 pub use self::{
-    body::CompressionBody, future::ResponseFuture, layer::CompressionLayer, service::Compression,
+    body::CompressionBody,
+    future::ResponseFuture,
+    layer::CompressionLayer,
+    predicate::{DefaultPredicate, Predicate},
+    service::Compression,
 };
 
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub(crate) enum Encoding {
-    #[cfg(feature = "compression-gzip")]
-    Gzip,
-    #[cfg(feature = "compression-deflate")]
-    Deflate,
-    #[cfg(feature = "compression-br")]
-    Brotli,
-    Identity,
-}
-
-impl Encoding {
-    fn to_str(self) -> &'static str {
-        match self {
-            #[cfg(feature = "compression-gzip")]
-            Encoding::Gzip => "gzip",
-            #[cfg(feature = "compression-deflate")]
-            Encoding::Deflate => "deflate",
-            #[cfg(feature = "compression-br")]
-            Encoding::Brotli => "br",
-            Encoding::Identity => "identity",
-        }
-    }
-
-    fn into_header_value(self) -> HeaderValue {
-        HeaderValue::from_static(self.to_str())
-    }
-
-    #[allow(unused_variables)]
-    fn parse(s: &str, accept: AcceptEncoding) -> Option<Encoding> {
-        match s {
-            #[cfg(feature = "compression-gzip")]
-            "gzip" if accept.gzip() => Some(Encoding::Gzip),
-            #[cfg(feature = "compression-deflate")]
-            "deflate" if accept.deflate() => Some(Encoding::Deflate),
-            #[cfg(feature = "compression-br")]
-            "br" if accept.br() => Some(Encoding::Brotli),
-            "identity" => Some(Encoding::Identity),
-            _ => None,
-        }
-    }
-
-    // based on https://github.com/http-rs/accept-encoding
-    fn from_headers(headers: &HeaderMap, accept: AcceptEncoding) -> Self {
-        let mut preferred_encoding = None;
-        let mut max_qval = 0.0;
-
-        for (encoding, qval) in encodings(headers, accept) {
-            if (qval - 1.0f32).abs() < 0.01 {
-                preferred_encoding = Some(encoding);
-                break;
-            } else if qval > max_qval {
-                preferred_encoding = Some(encoding);
-                max_qval = qval;
-            }
-        }
-
-        preferred_encoding.unwrap_or(Encoding::Identity)
-    }
-}
-
-// based on https://github.com/http-rs/accept-encoding
-fn encodings(headers: &HeaderMap, accept: AcceptEncoding) -> Vec<(Encoding, f32)> {
-    headers
-        .get_all(header::ACCEPT_ENCODING)
-        .iter()
-        .filter_map(|hval| hval.to_str().ok())
-        .flat_map(|s| s.split(',').map(str::trim))
-        .filter_map(|v| {
-            let mut v = v.splitn(2, ";q=");
-
-            let encoding = match Encoding::parse(v.next().unwrap(), accept) {
-                Some(encoding) => encoding,
-                None => return None, // ignore unknown encodings
-            };
-
-            let qval = if let Some(qval) = v.next() {
-                let qval = match qval.parse::<f32>() {
-                    Ok(f) => f,
-                    Err(_) => return None,
-                };
-                if qval > 1.0 {
-                    return None; // q-values over 1 are unacceptable
-                }
-                qval
-            } else {
-                1.0f32
-            };
-
-            Some((encoding, qval))
-        })
-        .collect::<Vec<(Encoding, f32)>>()
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -177,14 +88,28 @@ mod tests {
     use flate2::read::GzDecoder;
     use http_body::Body as _;
     use hyper::{Body, Error, Request, Response, Server};
+    use std::sync::{Arc, RwLock};
     use std::{io::Read, net::SocketAddr};
     use tokio::io::AsyncWriteExt;
     use tower::{make::Shared, service_fn, Service, ServiceExt};
 
+    // Compression filter allows every other request to be compressed
+    #[derive(Clone)]
+    struct Always;
+
+    impl Predicate for Always {
+        fn should_compress<B>(&self, _: &http::Response<B>) -> bool
+        where
+            B: http_body::Body,
+        {
+            true
+        }
+    }
+
     #[tokio::test]
     async fn works() {
         let svc = service_fn(handle);
-        let mut svc = Compression::new(svc);
+        let mut svc = Compression::new(svc).compress_when(Always);
 
         // call the service
         let req = Request::builder()
@@ -291,4 +216,68 @@ mod tests {
     async fn handle(_req: Request<Body>) -> Result<Response<Body>, Error> {
         Ok(Response::new(Body::from("Hello, World!")))
     }
+
+    #[tokio::test]
+    async fn will_not_compress_if_filtered_out() {
+        use predicate::Predicate;
+
+        const DATA: &str = "Hello world uncompressed";
+
+        let svc_fn = service_fn(|_| async {
+            let resp = Response::builder()
+                // .header("content-encoding", "br")
+                .body(Body::from(DATA.as_bytes()))
+                .unwrap();
+            Ok::<_, std::io::Error>(resp)
+        });
+
+        // Compression filter allows every other request to be compressed
+        #[derive(Default, Clone)]
+        struct EveryOtherResponse(Arc<RwLock<u64>>);
+
+        impl Predicate for EveryOtherResponse {
+            fn should_compress<B>(&self, _: &http::Response<B>) -> bool
+            where
+                B: http_body::Body,
+            {
+                let mut guard = self.0.write().unwrap();
+                let should_compress = *guard % 2 != 0;
+                *guard += 1;
+                dbg!(should_compress)
+            }
+        }
+
+        let mut svc = Compression::new(svc_fn).compress_when(EveryOtherResponse::default());
+        let req = Request::builder()
+            .header("accept-encoding", "br")
+            .body(Body::empty())
+            .unwrap();
+        let res = svc.ready().await.unwrap().call(req).await.unwrap();
+
+        // read the uncompressed body
+        let mut body = res.into_body();
+        let mut data = BytesMut::new();
+        while let Some(chunk) = body.data().await {
+            let chunk = chunk.unwrap();
+            data.extend_from_slice(&chunk[..]);
+        }
+        let still_uncompressed = String::from_utf8(data.to_vec()).unwrap();
+        assert_eq!(DATA, &still_uncompressed);
+
+        // Compression filter will compress the next body
+        let req = Request::builder()
+            .header("accept-encoding", "br")
+            .body(Body::empty())
+            .unwrap();
+        let res = svc.ready().await.unwrap().call(req).await.unwrap();
+
+        // read the compressed body
+        let mut body = res.into_body();
+        let mut data = BytesMut::new();
+        while let Some(chunk) = body.data().await {
+            let chunk = chunk.unwrap();
+            data.extend_from_slice(&chunk[..]);
+        }
+        assert!(String::from_utf8(data.to_vec()).is_err());
+    }
 }
